/*****************************************************************************
 *   Copyright (C) 2010 seb@exse.net										 *
 *   Copyright (C) 2011 Christian Brandtner brandtner@brandtner.net          *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *****************************************************************************/

/*28.8.2011 Parameter Waypoints */
/*14.6.2011 Parameter erweitert Wi232 Cebra */
/*23.8.2011 Parameter für Lastposition und Initialisierung Wi232, BT */

#include <avr/io.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include <stdlib.h>
#include <string.h>
#include <avr/eeprom.h>
#include <stdbool.h>
#include "lcd.h"
#include "main.h"
#include "timer.h"
#include "eeprom.h"
#include "Wi232.h"
#include "mk-data-structs.h"


#define EEpromVersion  0xE3

#define NumberOfWaypoints	55	//Anzahl der Waypoints in der EEPromliste
#define NumberOfWPLists		5	//Anzahl WP Listen im PKT

//*****************************************************************************
// 

uint8_t EE_LowBat   		EEMEM= 137;
uint8_t EE_DisplayTimeout 	EEMEM= 0;
uint8_t EE_DisplayLanguage 	EEMEM= 0;
uint8_t EE_WiTXRXChannel 	EEMEM= 1;
uint8_t EE_WiNetworkGroup 	EEMEM=66;
uint8_t EE_WiNetworkMode 	EEMEM=NetMode_Normal;
uint8_t EE_WiTXTO			EEMEM=TWaitTime16;
uint8_t EE_WiUartMTU		EEMEM=UartMTU64;
uint8_t EE_LCD_Orientation	EEMEM=0;
uint8_t EE_LCD_DisplayMode 	EEMEM=0;
uint8_t EE_LCD_Kontrast		EEMEM=27;
uint8_t EE_LCD_Helligkeit 	EEMEM=30;
uint8_t EE_USBBT 			EEMEM=0;
uint8_t EE_U02SV2 			EEMEM=0;
uint8_t EE_Debug 			EEMEM=0;
uint8_t	EE_WiIsSet			EEMEM=false;
uint8_t	EE_BTIsSet			EEMEM=false;
uint8_t	EE_BTPin[]			EEMEM= {'1','2','3','4'};
uint8_t	EE_BTName[]			EEMEM= {'C','B','R','A'};
uint32_t EE_LastLongitude	EEMEM=0;
uint32_t EE_LastLatitude	EEMEM=0;

WPListDirectory	   	EEWPDirectory[NumberOfWPLists] 		EEMEM;
WayPoints  			EEWayPointList[NumberOfWaypoints]   EEMEM;


uint8_t EE_PKTVersion		EEMEM=EEpromVersion;


volatile uint8_t DisplayTimeout;
volatile uint8_t DisplayLanguage;
volatile uint8_t WiTXRXChannel;
volatile uint8_t WiNetworkGroup;
volatile uint8_t WiNetworkMode;
volatile uint8_t WiTXTO;
volatile uint8_t WiUartMTU;
volatile uint8_t LCD_ORIENTATION;
volatile uint8_t LCD_DisplayMode;
volatile uint8_t LCD_Kontrast;
volatile uint8_t LCD_Helligkeit;
volatile uint8_t USBBT;
volatile uint8_t U02SV2;
volatile uint8_t Debug;
volatile uint8_t WiIsSet;				// Wi232 ist initialisiert
volatile uint8_t BTIsSet;				// BT ist initialisiert
volatile char	bt_pin[4];			//BT Pinnummer
volatile char	bt_name[4];			//BT Name
volatile uint32_t LastLongitude;	// Letzte Position
volatile uint32_t LastLatitude;
volatile WayPoints PKTWayPoint;		// Waypointdaten für einen Waypoint
volatile WPListDirectory PKTWayPointDirectory; // Inhaltsverzeichnis der Listen



void WriteWiInitFlag(void)
{
	eeprom_write_byte(&EE_WiIsSet, true);
}

void WriteBTInitFlag(void)
{
	eeprom_write_byte(&EE_BTIsSet, true);
}


void ReadLastPosition(void)

{
	LastLongitude = eeprom_read_dword(&EE_LastLongitude);
	LastLatitude = eeprom_read_dword(&EE_LastLatitude);
}

void WriteLastPosition(uint32_t ELongitude,uint32_t ELatitude)

{

	eeprom_write_dword(&EE_LastLongitude,ELongitude);
	eeprom_write_dword(&EE_LastLatitude,ELatitude);
}


//*****************************************************************************
// 
void ReadParameter (void)
{

	if (eeprom_read_byte(&EE_PKTVersion)== EEpromVersion)

	{
		MK_LowBat = eeprom_read_byte(&EE_LowBat);
		DisplayTimeout = eeprom_read_byte(&EE_DisplayTimeout);
		DisplayLanguage = eeprom_read_byte(&EE_DisplayLanguage);
		WiTXRXChannel =eeprom_read_byte(&EE_WiTXRXChannel);
		WiNetworkGroup =eeprom_read_byte(&EE_WiNetworkGroup);
		WiNetworkMode =eeprom_read_byte(&EE_WiNetworkMode);
		WiTXTO =eeprom_read_byte(&EE_WiTXTO);
		WiUartMTU =eeprom_read_byte(&EE_WiUartMTU);
		LCD_ORIENTATION = eeprom_read_byte(&EE_LCD_Orientation);
		LCD_DisplayMode = eeprom_read_byte(&EE_LCD_DisplayMode);
		LCD_Kontrast=eeprom_read_byte(&EE_LCD_Kontrast);
		LCD_Helligkeit=eeprom_read_byte(&EE_LCD_Helligkeit);
		USBBT = eeprom_read_byte(&EE_USBBT);
		U02SV2 = eeprom_read_byte(&EE_U02SV2);
		Debug = eeprom_read_byte(&EE_U02SV2);
		WiIsSet = eeprom_read_byte(&EE_WiIsSet);
		BTIsSet = eeprom_read_byte(&EE_BTIsSet);
//		LastLongitude = 0;					/* Beim Start immer löschen*/
//		LastLatitude = 0;					/* Beim Start immer löschen*/
	}

	else Delete_EEPROM();


}

//*****************************************************************************
void Delete_EEPROM(void)
{  /* EEPROM auf Default setzen*/

		lcd_cls ();
		lcd_printp (PSTR("EEPROM Parameter\r\n"), 0);
		lcd_printp (PSTR("auf Default setzen\r\n"), 0);
		MK_LowBat = 137;  				/*13,7V*/
		DisplayTimeout = 0; 			/* Display immer an */
		DisplayLanguage = 0; 			/* deutsch*/
		WiTXRXChannel = 1; 				/* Kanal 1 MK Standard*/
		WiNetworkGroup = 66; 			/*Gruppe 66  MK Standard*/
		WiNetworkMode = NetMode_Normal; 	/* MK Standard */
		WiTXTO	=TWaitTime16;			/* MK Standard*/
		WiUartMTU=	UartMTU64;			/* MK Standard*/
		LCD_ORIENTATION = 0; 			/* normale Ansicht */
		LCD_DisplayMode = 0;			/* Normal */
		LCD_Kontrast = 20;				/*Kontrast normal */
		LCD_Helligkeit = 30;			/*Helligkeit*/
		USBBT = 0;						/*USB Betrieb*/
		U02SV2 = 0;						/* Wi232 Standard*/
		Debug = 0;						/*kein Debug*/
		WiIsSet = false;				/*Flag für die Initialisierung  Wi232*/
		BTIsSet = false;				/*Flag für die Initialisierung  Bluetooth*/
		eeprom_write_byte(&EE_PKTVersion,EEpromVersion);
		WriteParameter();
		lcd_printp (PSTR("Waypoints loeschen\r\n"), 0);
		EEWayPointList_Clear();

		lcd_printp (PSTR("fertig\r\n"), 0);
		lcd_printp (PSTR("weiter mit ENTER\r\n"), 0);
		while (!(get_key_press (1 << KEY_ENTER)));
		return;

}

//*****************************************************************************

// 
void WriteParameter (void)
{
	 eeprom_write_byte(&EE_LowBat, MK_LowBat);
	 eeprom_write_byte(&EE_DisplayTimeout, DisplayTimeout);
	 eeprom_write_byte(&EE_DisplayLanguage, DisplayLanguage);
	 eeprom_write_byte(&EE_WiTXRXChannel, WiTXRXChannel);
	 eeprom_write_byte(&EE_WiNetworkGroup, WiNetworkGroup);
	 eeprom_write_byte(&EE_WiNetworkMode, WiNetworkMode);
	 eeprom_write_byte(&EE_WiTXTO, WiTXTO);
	 eeprom_write_byte(&EE_WiUartMTU, WiUartMTU);
	 eeprom_write_byte(&EE_LCD_Orientation, LCD_ORIENTATION);
	 eeprom_write_byte(&EE_LCD_DisplayMode, LCD_DisplayMode);
	 eeprom_write_byte(&EE_LCD_Kontrast, LCD_Kontrast);
	 eeprom_write_byte(&EE_LCD_Helligkeit, LCD_Helligkeit);
	 eeprom_write_byte(&EE_USBBT, USBBT);
	 eeprom_write_byte(&EE_U02SV2, U02SV2);
	 eeprom_write_byte(&EE_Debug, Debug);

}


//WPListDirectory	   	EEWPDirectory[NumberOfWPLists] 		EEMEM;
//WayPoints  			EEWayPointList[NumberOfWaypoints]   EEMEM;



void EEWayPointList_Clear(void)			// löschen der Waypointliste im EEProm
{
	uint8_t i;
	PKTWayPoint.Waypoint.Position.Latitude = 0;
	PKTWayPoint.Waypoint.Position.Longitude = 0;
	PKTWayPoint.Waypoint.Position.Altitude = 0;
	PKTWayPoint.Waypoint.Heading = 361;

	for(i = 0; i < MAX_WPLIST_LEN; i++)
	{
	PKTWayPointDirectory.WPList.WPDirectory[i] = 0;
	}



	for(i = 0; i < NumberOfWaypoints; i++)
	{
		lcd_printp (PSTR("."), 0);
		eeprom_write_byte (&EEWayPointList[i].WPIndex,i);
		eeprom_write_byte (&EEWayPointList[i].Waypoint.Position.Status,INVALID);
		eeprom_write_block ((const void*)&PKTWayPoint.Waypoint.Position.Latitude,(void*)&EEWayPointList[i].Waypoint.Position.Latitude,sizeof(EEWayPointList[i].Waypoint.Position.Latitude));
		eeprom_write_block ((const void*)&PKTWayPoint.Waypoint.Position.Longitude,(void*)&EEWayPointList[i].Waypoint.Position.Longitude,sizeof(EEWayPointList[i].Waypoint.Position.Longitude));
		eeprom_write_block ((const void*)&PKTWayPoint.Waypoint.Position.Altitude,(void*)&EEWayPointList[i].Waypoint.Position.Altitude,sizeof(EEWayPointList[i].Waypoint.Position.Altitude));
		eeprom_write_block ((const void*)&PKTWayPoint.Waypoint.Heading,(void*)&EEWayPointList[i].Waypoint.Heading,sizeof(EEWayPointList[i].Waypoint.Heading));

		eeprom_write_byte (&EEWayPointList[i].Waypoint.ToleranceRadius,0);		// in meters, if the MK is within that range around the target, then the next target is triggered
		eeprom_write_byte (&EEWayPointList[i].Waypoint.HoldTime,0);			// in seconds, if the was once in the tolerance area around a WP, this time defines the delay before the next WP is triggered
		eeprom_write_byte (&EEWayPointList[i].Waypoint.Type,POINT_TYPE_INVALID);
		eeprom_write_byte (&EEWayPointList[i].Waypoint.Event_Flag,0);			// future implementation
		eeprom_write_byte (&EEWayPointList[i].Waypoint.AltitudeRate,0);		// no change of setpoint
	}
	for(i = 0; i < NumberOfWPLists; i++)
	{
		lcd_printp (PSTR("."), 0);
		eeprom_write_byte (&EEWPDirectory[i].WPList.WPListnumber,i);
		eeprom_write_byte (&EEWPDirectory[i].WPList.WPListAktiv,false);
		eeprom_write_byte (&EEWPDirectory[i].WPList.POI_CAM_NICK_CTR,0);
		eeprom_write_byte (&EEWPDirectory[i].WPList.UsePOI,0);
		eeprom_write_block ((const void*)&PKTWayPointDirectory.WPList.WPDirectory,(void*)&EEWPDirectory[i].WPList.WPDirectory,sizeof(EEWPDirectory[i].WPList.WPDirectory));

	}
	lcd_printp (PSTR("\r\n"), 0);

}
